/**
 * WagbyDesigner アプリケーションコントローラークラス
 */
define([
  "dojo/_base/array",
  "dojo/_base/declare",
  "dojo/_base/event",
  "dojo/_base/fx",
  "dojo/_base/kernel",
  "dojo/_base/lang",
  "dojo/dom",
  "dojo/dom-attr",
  "dojo/dom-construct",
  "dojo/dom-style",
  "dojo/has",
  "dojo/i18n!../../wagbydesigner/nls/resource.js",
  "dojo/json",
  "dojo/on",
  "dojo/query",
  "dojo/request",
  "dojo/window",
  "dijit/ConfirmDialog",
  "dijit/Tooltip",
  "dijit/form/Select",
  "dijit/layout/BorderContainer",
  "dijit/layout/ContentPane",
  "dijit/registry",
  "dgrid/CellSelection",
  "dgrid/extensions/DnD",
  "dgrid/Grid",
  "dgrid/Editor",
  "dgrid/Keyboard",
  "dgrid/OnDemandGrid",
  "dgrid/Selection",
  "dgrid/Selector",
  "dstore/Memory",
  "dstore/Trackable",
  "wb/BrowseUserDefinedItemDialog",
  "wb/FunctionReferenceDialog",
  "wb/MenuPane",
  "wb/ModelPane",
  "wb/ModelReferenceTypeDetailDialog",
  "wb/ModelReferenceTypeDetailRepository",
  "wb/ModelRepository",
  "wb/ModelTabPane",
  "wb/ModelitemBulkUpdateDialog",
  "wb/ModelitemDetailDialog",
  "wb/ModelitemDetailRepository",
  "wb/PaneManager",
  "wb/PrimitiveTypeDetailDialog",
  "wb/PrimitiveTypeDetailRepository",
  "wb/Project",
  "wb/ProjectEnvironmentPane",
  "wb/BuildPane",
  "wb/OperationsPane",
  "wb/ScreenResourceID",
  "wb/ScriptEditor",
  "wb/TotalviewMeasureDialog",
  "wb/UserDefinedItemDialog",
  "wb/UserDefinedItemSettingRepository",
  "wb/WidgetDefault"
], function(
  array,
  declare,
  baseEvent,
  fx,
  kernel,
  lang,
  dom,
  domAttr,
  domConstruct,
  domStyle,
  has,
  jfcRes,
  JSON,
  on,
  query,
  request,
  win,
  ConfirmDialog,
  Tooltip,
  Select,
  BorderContainer,
  ContentPane,
  registry,
  CellSelection,
  DnD,
  Grid,
  Editor,
  Keyboard,
  OnDemandGrid,
  Selection,
  Selector,
  Memory,
  Trackable,
  BrowseUserDefinedItemDialog,
  FunctionReferenceDialog,
  MenuPane,
  ModelPane,
  ModelReferenceTypeDetailDialog,
  ModelReferenceTypeDetailRepository,
  ModelRepository,
  ModelTabPane,
  ModelitemBulkUpdateDialog,
  ModelitemDetailDialog,
  ModelitemDetailRepository,
  PaneManager,
  PrimitiveTypeDetailDialog,
  PrimitiveTypeDetailRepository,
  Project,
  ProjectEnvironmentPane,
  BuildPane,
  OperationsPane,
  ScreenResourceID,
  ScriptEditor,
  TotalviewMeasureDialog,
  UserDefinedItemDialog,
  UserDefinedItemSettingRepository,
  WidgetDefault
) {
  return declare(null, {
    constructor: function(releaseTimestamp) {
      // アプリケーション名
      this.APPNAME = "wagbydesigner";
      // URL
      this.BASEURL = "service";
      // テンプレートの HTML キャッシュ更新のためリリース日付を保持しておく
      this.RELEASE_TIMESTAMP = releaseTimestamp;
      // レイアウト設定済みフラグ
      this.layoutSettingChanged = false;
      this.jfcRes = jfcRes;
      this.locale = kernel.locale;
      this.wdVersion;
      this.FA_URL = "https://fontawesome.com/icons?d=gallery&m=free";
      this.SCRIPT_DEFINED_MARK = "* ";
      this.settings = {};
      this.principalData = null;
      this.DOMAIN_SYSTEM_ID = "1";
      this.DOMAIN_COMMON_ID = "2";
      this.DEFAULT_DIALOG_HEIGHT = 600;
    },

    initScreen: function() {
      // レイアウト枠設定
      this.borderContainer = new BorderContainer({
      }, "windowBorderContainer");
      // ヘッダ領域設定
      var headerContent = new ContentPane({
        region: "top"
      }, "toparea");
      var that = this;
      headerContent.on("Show", function() {
        that.setWagbyDesignerMenu();
      });
      this.borderContainer.addChild(headerContent);

      // コンテンツ領域設定
      this.borderContainer.addChild(new ContentPane({
        region: "center",
        style: "margin: 0; padding: 10px;"
      }, "centerarea"));
      // 対象ドメイン選択肢表示エリア
      this.targetDomainSelectArea = dom.byId("jfcTargetDomainSelectArea");
      // 対象ドメイン選択リストボックス生成
      this.createTargetDomainSelect();
      // 各コンテンツ設定(モデル一覧、モデル項目、プロジェクト設定)
      this.initPane();
      // レイアウト枠初期化処理(内部のウィジェット等の表示が初期化される)
      this.borderContainer.startup();
    },

    fadeInLoader: function() {
      domStyle.set("loader", "opacity", "0.9");
      domStyle.set("loader", "display", "");
    },

    fadeOutLoader: function() {
      fx.fadeOut({
        node: "loader",
        duration: 800,
        onEnd: function() {
          domStyle.set("loader", "display", "none");
        }
      }).play();
    },

    isLoaderDisplay: function() {
      return domStyle.get("loader", "display") !== "none";
    },

    setWagbyDesignerMenu: function() {
      // WagbyDesigner メニューボタンのラベル設定
      this.setMenuLabel();
      // WagbyDesigner メニューボタンのツールチップ設定
      this.setTooltip();
    },

    setMenuLabel: function() {
      domAttr.set("showModelPaneButton", "innerHTML", this.jfcRes.LABEL_MENU_Model);
      domAttr.set("showMenuPaneButton", "innerHTML", this.jfcRes.LABEL_MENU_Menu);
      domAttr.set("showEnvironmentPaneButton", "innerHTML", this.jfcRes.LABEL_MENU_ProjectEnvironment);
      domAttr.set("showBuildPaneButton", "innerHTML", this.jfcRes.LABEL_MENU_Build);
      domAttr.set("showOperationsPaneButton", "innerHTML", this.jfcRes.LABEL_MENU_Operation);
      domAttr.set("logoffButton", "innerHTML", this.jfcRes.LABEL_MENU_Logoff);
      domAttr.set("label_jfcTargetDomainSelect", "innerHTML", this.jfcRes.LABEL_jfcTargetDomainSelect);
    },

    setTooltip: function() {
      new Tooltip({
        connectId: ["menuButtonModel"],
        showDelay: 10,
        hideDelay: 10,
        label: this.jfcRes.LABEL_MENU_TOOLTIP_Model, //"モデルの定義を行ないます"
        position: ["below"],
      });
      this.menuTooltip = new Tooltip({
        connectId: ["menuButtonMenu"],
        showDelay: 10,
        hideDelay: 10,
        label: this.jfcRes.LABEL_MENU_TOOLTIP_Menu, //"メニューの定義を行ないます"
        position: ["below"],
      });
      new Tooltip({
        connectId: ["menuButtonProjectEnvironment"],
        showDelay: 10,
        hideDelay: 10,
        label: this.jfcRes.LABEL_MENU_TOOLTIP_ProjectEnvironment, //"環境設定を行ないます"
        position: ["below"],
      });
      new Tooltip({
        connectId: ["menuButtonBuild"],
        showDelay: 10,
        hideDelay: 10,
        label: this.jfcRes.LABEL_MENU_TOOLTIP_Build, //"アプリケーションのビルドを行ないます"
        position: ["below"],
      });
      new Tooltip({
        connectId: ["menuButtonOperations"],
        showDelay: 10,
        hideDelay: 10,
        label: this.jfcRes.LABEL_MENU_TOOLTIP_Operation, //"定義情報のバックアップ・リストアを行ないます"
        position: ["below"],
      });
    },

    initPane: function() {
      // ペイン表示・非表示を管理するマネージャオブジェクト
      this.paneManager = new PaneManager();

      // 環境
      this.projectEnvironmentPane = new ProjectEnvironmentPane({
        name: "projectEnvironment"
      });
      this.paneManager.addPane(this.projectEnvironmentPane);

      // モデルタブ
      this.modelTabPane = new ModelTabPane({
        name: "modelTab",
        href: "wb/paneTemplate/model/modelTab.html",
      });
      this.paneManager.addPane(this.modelTabPane);

      var modelRepository = new ModelRepository();
      // モデル
      this.modelPane = new ModelPane({
        name: "model",
        repositoryKeys: modelRepository.modelRepositoryKeys,
        href: "wb/paneTemplate/model/model.html",
      });
      this.paneManager.addPane(this.modelPane);

      // メニュー
      this.menuPane = new MenuPane({
        name: "menu"
      });
      this.paneManager.addPane(this.menuPane);

      // ビルド
      this.buildPane = new BuildPane({
        name: "build",
        href: "wb/paneTemplate/build/build.html",
      });
      this.paneManager.addPane(this.buildPane);

      // 運用
      this.operationsPane = new OperationsPane({
        name: "operations",
        href: "wb/paneTemplate/operations/operationPane.html",
      });
      this.paneManager.addPane(this.operationsPane);
      // メニューボタンイベント
      this.setMenuButtonEvent();
    },

    init: function() {
      this.getSettigns();
      this.screenResourceID = new ScreenResourceID();
      // プロジェクト
      this.project = new Project();
      // 画面設定
      this.initScreen();
      // ウィジェットデフォルト
      this.widgetDefault = new WidgetDefault(this.jfcRes, this.locale);
      // ダイアログの初期化
      this.initDialog(this.jfcRes);
      this.initModelitemTypeMap();
      this.initMasterModelMap();
      // ブラウザウィンドウ変更時のイベントをセットする
      this.setWindowResizeEvent();
      // 関連図のモデルデータ再生成フラグ
      this.relationModelDataNeedsRefresh = true;
      // ブラウザ判別
      this.setBrowserType();
      // ビルドメニューボタン
      this.btnShowBuildPane = dom.byId("showBuildPaneButton");
    },

    setWindowResizeEvent: function() {
      on(window, "resize", this.getWindowResizeFunc(this.borderContainer));
    },

    getWindowResizeFunc: function(borderContainer) {
      var that = this;
      var timer = false;
      return function() {
        if (timer) {
          clearTimeout(timer);
        }
        timer = setTimeout(function() {
          that.paneManager.needsLayoutPane = true;
        }, 500);
      };
    },

    setMenuButtonEvent: function() {
    },

    getDialogHeight: function() {
      var screenHeight = window.innerHeight;
      if (screenHeight > this.DEFAULT_DIALOG_HEIGHT) {
        return this.DEFAULT_DIALOG_HEIGHT;
      }
      // ブラウザウィンドウの高さが600未満の場合、ダイアログ内にスクロールバーが
      // 二重に表示されないようダイアログの高さを縮める
      return screenHeight - 70;
    },

    initDialog: function(jfcRes) {
      var dialogHeight = this.getDialogHeight();
      this.browseUserDefinedItemDialog = new BrowseUserDefinedItemDialog({
        name: "browseUserDefinedItem",
        title: jfcRes.TITLE_UserDefinedItemSetting_DIALOG, // ユーザ定義項目設定
        size: {width: 900, height: dialogHeight},
        displayCancelButton: false,
        jfcRes: jfcRes,
      });

      this.functionReferenceDialog = new FunctionReferenceDialog({
        name: "functionReference",
        title: jfcRes.TITLE_FUNCTION_REFERENCE_DIALOG, // 関数入力
        size: {width: 600, height: 390},
        displayCancelButton: true,
        jfcRes: jfcRes,
      });

      this.modelitemBulkUpdateDialog = new ModelitemBulkUpdateDialog({
        name: "modelitemBulkUpdate",
        title: jfcRes.TITLE_MODELITEM_BULKUPDATE_DIALOG, // モデル項目詳細一括設定
        size: {width: 900, height: dialogHeight},
        displayCancelButton: true,
        jfcRes: jfcRes,
      });

      var modelitemDetailRepository = new ModelitemDetailRepository();
      // 項目詳細定義ダイアログ
      this.modelitemDetailDialog = new ModelitemDetailDialog({
        name: "modelitemDetail",
        title: jfcRes.TITLE_MODELITEM_DETAIL_DIALOG, // "モデル項目詳細定義",
        size: {width: 770, height: dialogHeight},
        repositoryKeys: modelitemDetailRepository.modelitemDetailRepositoryKeys,
        jfcRes: jfcRes,
      });

      // 項目の型ダイアログ(他のストアモデルの参照)
      var modelReferenceTypeDetailRepository = new ModelReferenceTypeDetailRepository();
      this.modelReferenceTypeDetailDialog = new ModelReferenceTypeDetailDialog({
        name: "modelReferenceTypeDetail",
        title: jfcRes.TITLE_TYPE_DETAIL_DIALOG, //"モデル項目の型詳細設定",
        size: {width: 550, height: 355},
        repositoryKeys: modelReferenceTypeDetailRepository.modelReferenceTypeDetailRepositoryKeys,
        displayCancelButton: true,
      });

      // 項目の型ダイアログ(文字列、数値、日付・時刻)
      var primitiveTypeDetailRepository = new PrimitiveTypeDetailRepository();
      this.primitiveTypeDetailDialog = new PrimitiveTypeDetailDialog({
        name: "primitiveTypeDetail",
        title: jfcRes.TITLE_TYPE_DETAIL_DIALOG, //"モデル項目の型詳細設定",
        size: {width: 500, height: 280},
        repositoryKeys: primitiveTypeDetailRepository.primitiveTypeDetailRepositoryKeys,
        displayCancelButton: true,
        jfcRes: jfcRes,
      });

      // 集計ビューメジャー定義ダイアログ
      this.totalviewMeasureDialog = new TotalviewMeasureDialog({
        name: "totalviewMeasureDialog",
        title: jfcRes.TITLE_ACTION_TOTALVIEW_MEASURE_DIALOG, // "メジャー項目定義"
        size: {width: 760, height: dialogHeight},
        repositoryKeys: modelitemDetailRepository.modelitemDetailRepositoryKeys,
      });
      this.totalviewMeasureDialog.dialog.set("style", "width: 760; height: 600; overflow-y: scroll;");

      var userDefinedItemSettingRepository = new UserDefinedItemSettingRepository();
      this.userDefinedItemDialog = new UserDefinedItemDialog({
        name: "userDefinedItem",
        title: jfcRes.TITLE_UserDefinedItemSetting_DIALOG, // ユーザ定義項目設定
        size: {width: 900, height: dialogHeight},
        repositoryKeys: userDefinedItemSettingRepository.userDefinedItemSettingRepositoryKeys,
        displayCancelButton: true,
        jfcRes: jfcRes,
      });
    },

    getModelitemDetailCategoryOptions: function() {
      return [
        {label: "", value: " ", selected: true},
        {label: this.jfcRes.LABEL_modelitemDetailRepository_Basic, value: "basic"},
        {label: this.jfcRes.LABEL_TAB_ModelitemDetailDialog_Detail, value: "detail"},
        {label: this.jfcRes.LABEL_TAB_ModelitemDetailDialog_Check, value: "check"},
        {label: this.jfcRes.LABEL_TAB_ModelitemDetailDialog_Input, value: "input"},
        {label: this.jfcRes.LABEL_TAB_ModelitemDetailDialog_Output, value: "output"},
        {label: this.jfcRes.LABEL_TAB_ModelitemDetailDialog_Search, value: "search"},
        {label: this.jfcRes.LABEL_TAB_ModelitemDetailDialog_Default, value: "default"},
        {label: this.jfcRes.LABEL_TAB_ModelitemDetailDialog_Implicit, value: "implicit"},
        {label: this.jfcRes.LABEL_TAB_ModelitemDetailDialog_Help, value: "help"},
        {label: this.jfcRes.LABEL_TAB_ModelitemDetailDialog_Other, value: "other"}
      ];
    },

    createModelRelationDiagram: function() {
      this.modelTabPane.modelRelationDiagramPaneContent.drawUserModels();
    },

    showModelPane: function(modelId) {
      var that = this;
      this.releaseLock(function() {
        that.sendRequest({
          'url': "service/model/" + modelId,
          'options': {
            'query': {
              'lock': true,
            },
            'method': "GET",
            'handleAs': "json",
          }
        }, function(data) {
          var model = that.project.getModel(modelId);
          if (!model) {
            model = that.project.addNewModel(data, function() {
              that.setModelPaneData(model, data);
            });
          } else {
            that.setModelPaneData(model, data);
          }
        }, function(error) {
          jfcUserModelListStandby.hide();
        });
      });
    },

    setModelPaneData: function(model, data) {
      this.activeModel = model;
      this.modelPane.clearDataSetFlag();
      this.modelPane.setModel(model);
      model.setEditable(data.editable === "true");
      var that = this;
      // モデルのリポジトリデータを更新する
      model.modifyRepositoryData(data, function() {
        // モデル項目のリポジトリデータを更新する
        model.getModelitemData({
          callback: function() {
            that.modelPane.modelitemPaneContent.setModelitemDataToStore();
            that.modelPane.modelOtherPaneContent.setRepositoryData();
            that.paneManager.show(wagbyDesigner.modelPane);
          },
        });
      });
    },

    showPane: function(pane) {
      if (!wagbyDesigner.confirmDiscardLayoutSetting()) {
        return;
      }
      this.showTargetDomainSelect(pane);
      this.paneManager.show(pane);
    },

    showTargetDomainSelect: function(pane) {
      if (!this.isMicroserviceEnabled()) {
        return;
      }
      // モデル、運用以外の画面で対象ドメインリストボックスを表示する
      this.setTargetDomainSelectDisplay(!(pane === this.modelTabPane || pane === this.operationsPane));
    },

    showModelTabPane: function() {
      if (!wagbyDesigner.confirmDiscardLayoutSetting()) {
        return;
      }
      // 戻り先のモデルIDをクリアする
      this.modelPane.setBackToModelId("");
      this.showPane(this.modelTabPane);
    },

    showProjectEnvironmentPane: function() {
      this.showPane(this.projectEnvironmentPane);
    },

    showMenuPane: function() {
      var that = this;
      this.showPane(this.menuPane, function() {
        that.menuPane.refreshTitlePane();
      });
    },

    showBuildPane: function() {
      this.showPane(this.buildPane);
    },

    showOperationsPane: function() {
      this.showPane(this.operationsPane);
    },

    logoff: function(logoutUrl) {
      if (!wagbyDesigner.confirmDiscardLayoutSetting()) {
        return;
      }
      if (this.menuPane.isMenuPaneDisplayed) {
        this.menuPane.sendMenu(function() {
          location.href = logoutUrl;
        });
      } else {
        location.href = logoutUrl;
      }
    },

    checkAllModelListRow: function() {
      this.changeRowsCheckState({
        id: "userModelListBody",
        name: "rowSelect",
        checked: true
      });
    },

    uncheckAllModelListRow: function() {
      this.changeRowsCheckState({
        id: "userModelListBody",
        name: "rowSelect",
        checked: false
      });
    },

    checkAllModelitemListRow: function() {
      this.changeRowsCheckState({
        id: "modelitemListBody",
        name: "rowSelect",
        checked: true
      });
    },

    checkAllLayoutListRow: function() {
      this.changeRowsCheckState({
        id: "groupTabListBody",
        name: "rowSelect",
        checked: true
      });
    },

    uncheckAllLayoutListRow: function() {
      this.changeRowsCheckState({
        id: "groupTabListBody",
        name: "rowSelect",
        checked: false
      });
    },

    changeRowsCheckState: function(params) {
      var target = "#" + params.id + " input[name*='" + params.name +"']";
      query(target).forEach(function(node){
        registry.byId(node.id).set("value", params.checked);
      });
    },

    checkAllRow: function() {
      this.changeRowSelectCheckState({checked: true});
    },

    uncheckAllRow: function() {
      this.changeRowSelectCheckState({checked: false});
    },

    changeRowSelectCheckState: function(params) {
      query("input[name*='rowSelect']").forEach(function(node){
        registry.byId(node.id).set("value", params.checked);
      });
    },

    sortModelitemByNo: function() {
      this.activeModel.sortModelitemByNo();
    },

    getSelectedRowIds: function() {
      var selectedRowIds = [];
      query(".rowSelect input:checked").forEach(function(node) {
        selectedRowIds.push(parseInt(registry.byId(node.id).get("value")));
      });
      return selectedRowIds;
    },

    checkAllHasCondtion: function(checkbox) {
      var checked = false;
      if (checkbox.get("value")) {
        checked = true;
      }
      query("#modelitemListBody input[name*='hasConditionModel']").forEach(function(node){
        registry.byId(node.id).set("value", checked);
      });
    },

    checkAllHasList: function(checkbox) {
      var checked = false;
      if (checkbox.get("value")) {
        checked = true;
      }
      query("#modelitemListBody input[name*='hasListModel']").forEach(function(node){
        registry.byId(node.id).set("value", checked);
      });
    },

    showModelitemTypeDetailDialog: function(modelitem) {
      // ダイアログを閉じた時にツールチップが表示されないように
      // 型のリストボックスにフォーカスを当てる
      if (modelitem.widgets["model/modelitem/@typeSimple"].widget) {
        modelitem.widgets["model/modelitem/@typeSimple"].widget.focus();
      }
      if (modelitem.getTypeSimple() === "referModel") {
        this.modelReferenceTypeDetailDialog.show(modelitem);
      } else {
        this.primitiveTypeDetailDialog.show(modelitem);
      }
    },

    isModelitemTypeContainer: function(typeSimple) {
      if (typeSimple === "container" || typeSimple === "containerId") {
        return true;
      }
      return false;
    },

    isIdrefMasterModel: function(idref) {
      if (this.masterModelMap[idref]) {
        return true;
      }
      return false;
    },

    showModelitemDetailDialog: function(modelitemId) {
      this.modelitemDetailDialog.show(
        this.activeModel.getModelitemById(modelitemId).detail);
    },

    stopClickEvent: function(widgetItems) {
      array.forEach(widgetItems, function(widgetItem) {
        if (dom.byId(widgetItem.widgetPlaceId)) {
          widgetItem.widget.on("click", function(event) {
            baseEvent.stop(event);
          });
        }
      });
    },

    showModelitemBulkUpdateDialog: function() {
      this.modelitemBulkUpdateDialog.show();
    },

    showFunctionReferenceDialog: function(widget) {
      if (!widget) {
        return;
      }
      this.functionReferenceDialog.show(this.activeModel.getId(), widget);
    },

    showDialog: function (message, option) {
      var that = this;
      var html = "";
      if (message) {
        html = message.replace(/\n/g, "<br>");
      }
      var dialog = registry.byId("messageDialog");
      if (dialog) {
        var content = "<div style='margin: 10px;'>"
        + "<div style='margin: 20px 0;'>" + html + "</div>"
        + "<div style='text-align: center;'>"
        + "<button data-dojo-type=\"dijit/form/Button\" onclick=\"messageDialog.hide();\">" + that.jfcRes.LABEL_BUTTON_CLOSE + "</button>" // 閉じる
        + "</div></div>";
        dialog.set("content", content);
        dialog.show();
        dialog.on("hide", function() {
          if (option && option.isReturnLogon) {
            location.href = "/wagbydesigner/logon.jsp";
          }
        });
      }
    },

    showFatalErrorDialog: function (message) {
      this.showDialog(message, {isReturnLogon: true});
    },

    showTotalviewMeasureDialog: function(no) {
      this.totalviewMeasureDialog.show(no);
    },

    // モデルIDのリポジトリデータを更新する
    modifyOtherModelModelData: function(modelId, modifyMapOtherModel) {
      var model = this.project.getModelByModelId(modelId);
      if (!model) {
        return;
      }
      if (model.modifyRepositoryData && modifyMapOtherModel.modelMap) {
        model.modifyRepositoryData(modifyMapOtherModel.modelMap);
        this.modelPane.updateActionPaneRepositoryData(modifyMapOtherModel.modelMap);
      }
      if (modifyMapOtherModel.modelitemMap) {
        this.modifyOtherModelModelitemData(model, modifyMapOtherModel.modelitemMap);
      }
    },

    // モデルIDのリポジトリデータを更新する
    modifyOtherModelModelitemData: function(model, modelitemMap) {
      for (modelitemName in modelitemMap) {
        var modelitem = model.getModelitemByName(modelitemName);
        if (modelitem) {
          modelitem.modifyRepositoryData(modelitemMap[modelitemName]);
        }
      }
    },

    // モデルID、モデル項目IDを変更時に参照されているリポジトリを修正した
    // データがサーバから送信されるので、モデル毎にリポジトリの更新を行なう
    modifyOtherModelData: function(modifyMapOtherModel) {
      for (modelId in modifyMapOtherModel) {
        if (modifyMapOtherModel[modelId].modelMap) {
          this.modifyOtherModelModelData(modelId, modifyMapOtherModel[modelId]);
        }
      }
    },

    // 設定変更に関連する自モデルの設定値変更を行なう
    modifyThisModelData: function(modelId, repositoryKey, modifyMap) {
      if (Object.keys(modifyMap).length === 0) {
        return;
      }
      var modelMap = {};
      for (var key in modifyMap) {
        if (key === repositoryKey) {
          continue;
        }
        modelMap[key] = modifyMap[key];
      }
      var modifyMapThisModel = {};
      modifyMapThisModel.modelMap = modelMap;
      this.modifyOtherModelModelData(modelId, modifyMapThisModel);
    },

    // 設定変更に関連する自モデルの設定値変更を行なう
    modifyThisModelitemData: function(modelId, modifyMap) {
      if (Object.keys(modifyMap).length === 0) {
        return;
      }
      this.modifyOtherModelModelData(modelId, {
        modelitemMap: modifyMap,
      });
    },

    updateUserDefinedItemSetting: function(userDefinedItemSetting) {
      this.modelTabPane.userDefinedItemSettingListPaneContent.updateUserDefinedItemSetting(userDefinedItemSetting);
    },

    initModelitemTypeMap: function() {
      this.typeDetailToTypeSimple = {
        "string": "string", // 文字列
        "zipcode": "string", // 郵便番号
        "mailaddress": "string", // メールアドレス
        "URL": "string",
        "filename": "string", // ファイル名
        "number": "number", // 整数
        "byte": "number", // 1バイト整数
        "short": "number", // 2バイト整数
        "integer": "number", // 4バイト整数
        "long": "number", // 8バイト整数
        "float": "number", // 4バイト浮動小数点
        "double": "number", // 8バイト浮動小数点
        "date": "dateTime", // 日付
        "dateTime": "dateTime", // 日付と時間
        "time": "dateTime", // 時間
        "referenceMaster_listbox": "referModel", // マスタモデルの参照(コンボボックス)
        "referenceMaster_radiobutton": "referModel", // マスタモデルの参照(ラジオボタン)
        "referenceMaster_checkbox": "referModel", // マスタモデルの参照(チェックボックス)
        "reference_listbox": "referModel", // 他のストアモデルの参照(コンボボックス)
        "reference_radiobutton": "referModel", // 他のストアモデルの参照(ラジオボタン)
        "reference_checkbox": "referModel", // 他のストアモデルの参照(チェックボックス)
        "reference_searchWindow": "referModel", // 他のストアモデルの参照(検索画面)
        "reference_lookup": "referModel", // 他のストアモデルの参照連動
        "container": "container", // (繰り返し項目のコンテナ)
        "containerId": "containerId", // (繰り返し項目コンテナ用のID)
        "createuserid": "string", // 固定値(作成したユーザのID)
        "updateuserid": "string", // 固定値(更新したユーザのID)
        "createdatetime": "dateTime", // 固定値(作成した日付と時間)
        "updatedatetime": "dateTime" // 固定値(更新した日付と時間)
      };

      this.typeSimpleDefaultDetailType = {
        "string": "string", // 文字列
        "number": "number", // 整数
        "dateTime": "date", // 日付
        "container": "container", // (繰り返し項目のコンテナ)
        "containerId": "containerId" // (繰り返し項目コンテナ用のID)
      };
    },

    initMasterModelMap: function() {
      this.masterModelMap = {
        "compulsoryChange_m": true, // juserのカスタマイズコードでマスタモデルとして扱っている。
        "jfcdispositiontype_m": true, // "ブラウザで表示", "ダウンロード"
        "jfcenable_m": true, // "有効"
        "jfclockforall_m": true, // "いいえ", "はい"
        "jfcmailtemplate_auth_m": true, // "利用しない", "利用する"
        "jfcmailtemplate_mailtype_m": true, // "テキストのみ", "書式付きテキストのみ", "ソースのみ", "テキストと書式付きテキスト", "テキストとHTMLソース", "EMLファイル"
        "jfcmailtemplate_protocol_m": true, // "SMTP", "SMTP over SSL"
        "jfcmailtemplate_popauthprotocol_m": true, // "POP3", "POP3 over SSL"
        "jfcmodelcreatetype_m": true, // "システム", "ユーザー"
        "jfcmodelkind_m": true, // "ストアモデル", "マスタモデル"
        "jfcmodeluseflag_m": true, // "有", "無"
        "jfcportletType_m": true, // ポートレットタイプ
        "jfcwfnodeType_m": true, // "順次", "合議"
        "jfcworkflow_mstype_m": true, // "新規登録", "申請", "承認", "却下", "保留", "差し戻し", "取り消し", "決裁"
        "jfcworkflow_remand_m": true, // "一つ前へ戻
        "jprincipal": true // プリンシパル
      };
    },

    createTargetDomainSelect: function() {
      var that = this;
      this.targetDomainSelect = new Select({
        "name": "jfcTargetDomainSelect",
        "onChange": function(value) {
          that.targetDomainChangeFunc(value);
        },
      }, "jfcTargetDomainSelect");
      this.targetDomainSelect.startup();
    },

    targetDomainChangeFunc: function(value) {
      if (value === undefined || !this.isMicroserviceEnabled()) {
        return;
      }
      this.projectEnvironmentPane.targetDomainChange(value);
      this.menuPane.targetDomainChange(value);
    },

    capFirst: function(str) {
      if (!str) {
        return "";
      }
      var name = this.convClassname4Relax(str);
      return name.substring(0, 1).toUpperCase() + name.substring(1);
    },

    convClassname4Relax: function(s) {
      if (!s) {
        return "";
      }
      var convStr = "";
      for (var i=0,l=s.length; i<l; i++) {
        var c = s.charAt(i);
        if (c == '_') {
          while (i+1 < s.length && (c = s.charAt(++i)) == '_') ;
          if (c != '_') {
            convStr += c.toUpperCase();
          }
        } else {
          convStr += c;
        }
      }
      return convStr;
    },

    // リストボックスの選択肢を作成する
    refreshSelectOptions: function(args) {
      var selectWidget = args.widget;
      if (!selectWidget) {
        return;
      }
      selectWidget.removeOption(selectWidget.getOptions());
      selectWidget.addOption(args.options);
      if (args.defaultValue) {
        selectWidget.set("_onChangeActive", false);
        selectWidget.set("value", args.defaultValue);
        selectWidget.set("_onChangeActive", true);
      }
    },

    checkItemReferenced: function(widgetItem, overwriteOptions) {
      var modifyMapOtherModel;
      var options = {
        headers: {
          'Content-Type': "application/x-www-form-urlencoded; charset=utf-8",
          'x-wd-check-only': true
        },
        sync: true,
        method: "PUT",
        handleAs: "json"
      };
      var url;
      if (widgetItem) {
        url = widgetItem.getUrl();
        var data = {};
        data[widgetItem.getRepositoryKey()] = widgetItem.widget.get("value");
        options.data = data;
      }
      if (overwriteOptions) {
        if (overwriteOptions.url) {
          url = overwriteOptions.url;
        }
        if (overwriteOptions.method) {
          options.method = overwriteOptions.method;
        }
      }
      request(url, options).then(function(data) {
        if (data.updateModelInfo && data.updateModelInfo.modifyMapOtherModel) {
          modifyMapOtherModel = data.updateModelInfo.modifyMapOtherModel;
        } else if (data.updateModelitemInfo && data.updateModelitemInfo.modifyMapOtherModel) {
          modifyMapOtherModel = data.updateModelitemInfo.modifyMapOtherModel;
        }
      }, function(err) {
        if (widgetItem) {
          widgetItem.errorOnSendData(err.response.data, widgetItem, widgetItem.widget.get("value"));
        }
      });
      return modifyMapOtherModel;
    },

    getReferencedItemMessage: function (modifyMapOtherModel) {
      var message = "";
      for (var modelId in modifyMapOtherModel) {
        message += wagbyDesigner.jfcRes.LABEL_MODEL_ID + ": " + modelId + "\n";
        message += this.getRelatedProps(modifyMapOtherModel[modelId].modelitemMap, this.getPropertyKeyHandlerFunc(), wagbyDesigner.jfcRes.LABEL_MODELITEM_ID);
        message += this.getRelatedProps(modifyMapOtherModel[modelId].scriptMap, this.getPropertyKeyValueHandlerFunc(), wagbyDesigner.jfcRes.LABEL_TAB_ScriptPane);
        message += "\n";
      }
      return message;
    },

    getRelatedProps: function(propMap, handlerFunc, label) {
      var props = [];
      for (var prop in propMap) {
        handlerFunc(prop, propMap, props);
      }
      if (props.length === 0) {
        return "";
      }
      return "  " + label + ": " + props.join(", ") + "\n";
    },

    getPropertyKeyHandlerFunc: function() {
      return function(prop, propMap, props) {
        props.push(prop);
      }
    },

    getPropertyKeyValueHandlerFunc: function() {
      return function(prop, propMap, props) {
        props.push(prop + ": " + propMap[prop]);
      }
    },

    // 参照設定されているモデルID、項目ID変更時の確認ダイアログメッセージ
    getReferencedItemChangeConfirmMessage: function (modifyMapOtherModel) {
      var message = this.jfcRes.MSG_CONFIRM_MODIFY_MODEL_MODELITEM; // この項目は下記のモデルで参照されています\n変更してもよろしいですか？\n(参照設定されている値も全て変更されます)
      message += this.getReferencedItemMessage(modifyMapOtherModel);
      return message;
    },

    getTypeLabel: function(type) {
      if (type === "string") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_string_1; // 文字列
      } else if (type === "zipcode") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_string_2; // 郵便番号
      } else if (type === "mailaddress") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_string_3; // メールアドレス
      } else if (type === "filename") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_string_5; // ファイル名
      } else if (type === "number") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_number_1; // 整数
      } else if (type === "byte") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_number_2; // 1バイト整数
      } else if (type === "short") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_number_3; // 2バイト整数
      } else if (type === "integer") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_number_4; // 4バイト整数
      } else if (type === "long") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_number_5; // 8バイト整数
      } else if (type === "float") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_number_6; // "4バイト浮動小数"
      } else if (type === "double") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_number_7; // "8バイト浮動小数"
      } else if (type === "date") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_dateTime_1; // "日付
      } else if (type === "dateTime") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_dateTime_2; // "日付と時刻"
      } else if (type === "time") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_dateTime_3; // "時刻"
      } else if (type.indexOf("listbox") !== -1) {
        return this.jfcRes.LABEL_INPUT_TYPE_SELECT; // "リストボックス"
      } else if (type.indexOf("radiobutton") !== -1) { // "ラジオボタン"
        return this.jfcRes.LABEL_INPUT_TYPE_RADIO; // "ラジオボタン"
      } else if (type.indexOf("checkbox") !== -1) { // "チェックボックス"
        return this.jfcRes.LABEL_INPUT_TYPE_CHECK; // "チェックボックス"
      } else if (type.indexOf("searchWindow") !== -1) { // "検索画面"
        return this.jfcRes.LABEL_INPUT_TYPE_SEARCH; // "検索画面"
      } else if (type.indexOf("lookup") !== -1) { // "参照連動"
        return this.jfcRes.LABEL_INPUT_TYPE_REF; // "参照連動"
      }
      return type;
    },

    sendRequest: function(properties, callback, errorHandler) {
      var that = this;
      var requestOptions = properties.options || {};
      if (!requestOptions.headers) {
        requestOptions.headers = {};
      }
      if (!requestOptions.headers['Content-Type']) {
        requestOptions.headers['Content-Type'] = "application/x-www-form-urlencoded; charset=utf-8";
      }
      // Dojo ライブラリで JSON データのパース処理を行なわないように
      // レスポンスデータの取り扱い方法についてのプロパティは変数へ
      // 保存して削除する
      var handleAs = requestOptions.handleAs;
      delete requestOptions['handleAs'];

      var url = properties.url;
      if (url.lastIndexOf(that.BASEURL, 0) === -1) {
        url = that.BASEURL + url;
      }
      return request(url, requestOptions).then(function(data) {
        if (data) {
          if (data.response && data.response.status === 0) {
            wagbyDesigner.checkServerConnection(err);
          } else if (data !== "" && handleAs && handleAs.toLowerCase() === "json") {
            // レスポンスデータの取り扱いが JSON の場合には、パース処理を行なう
            if (data.indexOf("<html>") !== -1) {
              // JSON データとして取り扱うデータのレスポンスが HTML の場合はエラーとする
              that.showFatalErrorDialog(that.jfcRes.MSG_ALERT_DISCONNECT_SERVER); // "WagbyDesigner サーバとの接続が切断されました<br/> ログオン画面を表示します"
              return;
            }
            try {
              data = JSON.parse(data);
            } catch(e) {
              console.error("WagbyDesigner.sendRequest JSON.parse error: " + e);
            }
          }
        }
        if (callback) {
          callback(data);
        }
      }, function(err) {
console.error("sendRequest Error: ", err);
        var message;
        if (errorHandler) {
          message = errorHandler(err) || "";
        } else {
          message = that.jfcRes.MSG_ERROR_SEND_REQUEST; // "リポジトリデータ処理中にエラーが発生しました"
        }
        if (err && err.response.status === 0) {
          wagbyDesigner.checkServerConnection(err);
        } else if (err && err.response.status >= 500 && err.response.status < 600) {
          if (err.response && err.response.data) {
            message += err.response.data;
          } else {
            console.log(err);
          }
        }
        if (message && message !== "") {
          wagbyDesigner.showDialog(message);
        }
      });
    },

    enableAllWidgets: function(widgets) {
      this.setWidgetDisabled(widgets, false);
    },

    disableAllWidgets: function(widgets) {
      this.setWidgetDisabled(widgets, true);
    },

    setWidgetDisabled: function(widgets, disabled) {
      array.forEach(widgets, function(widget) {
        widget.set("disabled", disabled);
      });
    },

    checkServerConnection: function(data) {
      if (data && data.response.status === 0) {
        this.showFatalErrorDialog(this.jfcRes.MSG_ALERT_DISCONNECT_SERVER); // "WagbyDesigner サーバとの接続が切断されました<br/> ログオン画面を表示します"
      }
    },

    openScriptEditor: function(widget, mode) {
      if (!this.scriptEditor) {
        this.aceEditor = dom.byId("aceEditor");
        this.scriptEditor = new ScriptEditor();
      }
      // ブラウザ画面の70%の大きさをエディタサイズとする
      var width = Math.floor(win.getBox().w * 0.7);
      var height = Math.floor(win.getBox().h * 0.7);
      domStyle.set(this.aceEditor, "height", height + "px");
      domStyle.set(this.aceEditor, "width", width + "px");
      this.scriptEditor.setScriptWidget(widget);
      this.scriptEditor.setMode(mode);
      scriptEditorDialog.show();
    },

    getProjectId: function() {
      return this.projectEnvironmentPane.getProjectId();
    },

    getTomcatHttpPort: function() {
      return this.projectEnvironmentPane.getTomcatHttpPort();
    },

    refreshProjectEnvironmentRepositoryItems: function() {
      this.projectEnvironmentPane.projectEnvironment.refreshRepositoryItems();
    },

    setScreenResources: function(screenId) {
      var resourceIDs = wagbyDesigner.screenResourceID.getResourceIDs(screenId);
      if (!resourceIDs) {
        console.warn("ScreenId:["+screenId+"] resource NOT FOUND");
        return;
      }
      var that = this;
      setTimeout(function() {
        for (var i=0,l=resourceIDs.length; i<l; i++) {
          var resourceID = resourceIDs[i];
          that.setScreenResource(resourceID.key, resourceID.id);
        }
      }, 1);
    },

    setScreenResource: function(key, id) {
      var that = this;
      if (!id) {
        id = key;
      }
      var node = dom.byId(id);
      if (node) {
        var defaultLabel = domAttr.get(node, "innerHTML");
//        if (defaultLabel !== that.jfcRes[key]) {
//          console.error("id:["+id+"] resource label different, defaultLabel:["+defaultLabel+"], res:["+that.jfcRes[key]+"]");
//        }
        domAttr.set(node, "innerHTML", that.jfcRes[key]);
      } else {
        console.warn("node:["+id+"] NOT FOUND");
      }
    },

    setTabColorError: function(tabContainer, errorTabIndex) {
      if (errorTabIndex <= 0 ||
        errorTabIndex > tabContainer.tablist.containerNode.childNodes.length) {
        return;
      }
      var errorTab = tabContainer.tablist.containerNode.childNodes[errorTabIndex - 1];
      if (!errorTab) {
        return;
      }
      domStyle.set(errorTab, "background", "#ffff66");
    },

    setTabColorNormal: function(tabContainer) {
      var tabs = tabContainer.tablist.containerNode.childNodes;
      for (var i=0,l=tabs.length; i<l; i++) {
        domStyle.set(tabs[i], "background", "");
      }
    },

    createConfirmDialog: function(args, callback) {
      var that = this;
      var confirmDialog;
      confirmDialog = new ConfirmDialog({
        title: args.title || "WagbyDesigner",
        style: args.style || "width: 350px",
        autofocus: false
      });
      domStyle.set(confirmDialog.actionBarNode, {
        'text-align': "right",
        'margin': "10px"
      });
      confirmDialog.okButton.set("label", args.okButtonLabel || that.jfcRes.LABEL_BUTTON_YES); // はい
      confirmDialog.cancelButton.set("label", args.cancelButtonLabel || that.jfcRes.LABEL_BUTTON_NO); // いいえ

      confirmDialog.on("Execute", function() {
        confirmDialog.onExecuteFunc();
      });
      confirmDialog.on("Cancel", function() {
        confirmDialog.onCancelFunc();
      });
      confirmDialog.on("Show", function() {
        confirmDialog.onShowFunc(confirmDialog);
      });
      confirmDialog.on("Hide", function() {
        confirmDialog.onHideFunc();
      });
      domStyle.set(confirmDialog.containerNode, {
        "overflow": "auto",
        "maxHeight": "350px",
        "whiteSpace": "pre",
      });
      var messageDom = domConstruct.create("span", { innerHTML: args.message || "", style: "user-select: all" }, confirmDialog.containerNode);
      confirmDialog.updateMessage = function(message) {
        messageDom.innerHTML = message || "";
      }
      confirmDialog.updateMessage(args.message || "");

      confirmDialog.getDialogSize = function() {
        return "width: " + confirmDialog.width + "px;";
      }

      confirmDialog.resizeDialog = function(size) {
        if (!size) {
          return;
        }
        if (size.width) {
          confirmDialog.width = size.width;
        }
        confirmDialog.set("style", confirmDialog.getDialogSize());
      }

      confirmDialog.setEventCallbackFunc = function(args) {
        confirmDialog.onExecuteFunc = args.onExecute || function(){};
        confirmDialog.onCancelFunc = args.onCancel || function(){};
        confirmDialog.onShowFunc = args.onShow || function(){};
        confirmDialog.onHideFunc = args.onHide || function(){};
      }
      if (args.size) {
        confirmDialog.resizeDialog(args.size);
      }

      confirmDialog.setEventCallbackFunc(args);
      if (callback) {
        callback(confirmDialog);
      }
      return confirmDialog;
    },

    isReferModelitemTypeZipcode: function(idref, rolecontent) {
      var model = this.project.getModelById(idref);
      if (!model) {
        return false;
      }
      // 参照先項目が参照連動の場合、参照先のモデルを確認する
      var modelitem = model.getModelitemByName(rolecontent);
      if (!modelitem) {
        return false;
      }
      if (modelitem.isReferModel()) {
        return this.isReferModelitemTypeZipcode(
          modelitem.getIdref(),
          modelitem.getRolecontent()
        );
      }
      var type = modelitem.getType();
      var addressfield = modelitem.getAddressfield();
      if (type === "zipcode" &&
          addressfield && (addressfield !== "" || addressfield !== " ")) {
        return true;
      }
      return false;
    },

    isChecked: function(value) {
      if (value === wagbyDesigner.jfcRes.OPTION_CHECKED) {
        return true;
      }
      return false;
    },

    bubbleInfoMessage: function(message, duration) {
      this.bubbleMessage(message, "info", duration);
    },

    bubbleErrorMessage: function(message) {
      this.bubbleMessage(message, "error");
    },

    bubbleMessage: function(message, type, duration) {
      if (!message || message === "") {
        return;
      }
      jfcMessageToaster.setContent(message, type, (duration || 4000));
      jfcMessageToaster.show();
    },

    setRelationModelDataNeedsRefresh: function(flag) {
      this.relationModelDataNeedsRefresh = flag;
    },

    getRelationModelDataNeedsRefresh: function() {
      return this.relationModelDataNeedsRefresh;
    },

    // テキスト入力欄にフォーカスをセットして入力文字列を選択状態にする
    focusAndSelect: function(textbox) {
  	if (!textbox) {
        return;
      }
      textbox.focus();
      setTimeout(function() {
        var length = 0;
        if (textbox.value) {
          length = textbox.value.length;
        }
        textbox.setSelectionRange(0, length);
      }, 1);
    },

    setLayoutSettingChanged: function(flag) {
      this.layoutSettingChanged = flag;
    },

    isLayoutSettingChanged: function() {
      return this.layoutSettingChanged;
    },

    confirmDiscardLayoutSetting: function() {
      if (wagbyDesigner.isLayoutSettingChanged() &&
          !confirm(this.jfcRes.MSG_CONFIRM_LAYOUT_SETTING_UNSAVED)) {
        return false;
      }
      this.setLayoutSettingChanged(false);
      return true;
    },

    openWagbyManualPage: function(page) {
      var url = "http://wagby.com/";
      if (this.locale && this.locale.toLowerCase().lastIndexOf("ja", 0) === -1) {
        url = url + "en/";
      }
      url = url + "manual8/";
      if (page) {
        url = url + page;
      }
      this.MANUAL_WINDOW = window.open(url, "MANUAL_WINDOW");
    },

    createEventSource: function(args) {
      var eventSource = new EventSource(args.url);
      if (!eventSource) {
        return null;
      }
      if (args.onOpen) {
        eventSource.addEventListener("open", function (event) {
          args.onOpen(event);
        });
      }
      eventSource.addEventListener("message", function (event) {
        if (args.onMessage) {
          args.onMessage(event, eventSource);
        }
        if (event.data === "EventSourceComplete") {
          if (args.onComplete) {
            args.onComplete(event);
          }
          eventSource.close();
        }
      });
      eventSource.addEventListener("error", function (event) {
        eventSource.close();
        if (args.onError) {
          args.onError(event);
        }
      });
      return eventSource;
    },

    setBrowserType: function() {
      this.isChrome = has("chrome");
      this.isFirefox = has("ff");
      this.isEdge = has("edge");
    },

    isMessageQueueEnabled: function() {
      return this.projectEnvironmentPane.isMessageQueueEnabled();
    },

    getDisplayitemSortFunc: function() {
      var reverseSortFlag = 1;
      if (!(this.isEdge || this.isChrome && this.isChrome >= 70)) {
        // ブラウザによってソート順が異なるため、Chrome 69 以前、
        // Edge 以外のブラウザの場合はソートを逆順にする
        reverseSortFlag = -1;
      }
      return function(a, b) {
        if (!a.order || !b.order ||
            (a.order === wagbyDesigner.jfcRes.OPTION_CHECKED && b.order === wagbyDesigner.jfcRes.OPTION_CHECKED) ||
            (a.order !== wagbyDesigner.jfcRes.OPTION_CHECKED && b.order === wagbyDesigner.jfcRes.OPTION_CHECKED)) {
          return 1 * reverseSortFlag;
        }
        if (a.order === wagbyDesigner.jfcRes.OPTION_CHECKED &&
            b.order !== wagbyDesigner.jfcRes.OPTION_CHECKED) {
          return -1 * reverseSortFlag;
        }
        try {
          var aOrder = parseInt(a.order, 10);
          var bOrder = parseInt(b.order, 10);
          return aOrder - bOrder;
        } catch(e) {
          console.error(e);
        }
        return 1 * reverseSortFlag;
      }
    },

    clearModelListCache: function() {
      this.modelTabPane.modelListPaneContent.clearCache();
    },

    releaseLock: function(callback) {
      this.sendRequest({
        'url': "/lock/release/thissession",
        'options': {
          'handleAs': 'json',
          'method': "POST",
        },
      }, function(data) {
        if (callback) {
          callback(data);
        }
      }, function(error) {
        jfcUserModelListStandby.hide();
      });
    },

    getOpenFontAwesomeWindowFunc: function() {
      var that = this;
      return function() {
        that.openFontAwesomeWindow();
      };
    },

    openFontAwesomeWindow: function() {
      if (this.FA_WINDOW && !this.FA_WINDOW.closed) {
        this.FA_WINDOW.focus();
        return;
      }
      this.FA_WINDOW = window.open(this.FA_URL, "FA_WINDOW");
    },

    setMarkSelectOption: function(selectWidget, selectedOptionValues) {
      var options = selectWidget.get("options");
      for (var i=0,l=options.length; i<l; i++) {
        var option = options[i];
        if (selectedOptionValues[option.value]) {
          this.setMarkToOptionLabel(selectWidget, option);
        } else {
          this.removeMarkFromOptionLabel(selectWidget, option);
        }
      }
    },

    setMarkToOptionLabel: function(selectWidget, option) {
      if (this.isMarkSetOptionLabel(option)) {
        return;
      }
      option.label = this.SCRIPT_DEFINED_MARK + option.label;
      selectWidget.updateOption(option);
    },

    removeMarkFromOptionLabel: function(selectWidget, option) {
      if (!this.isMarkSetOptionLabel(option)) {
        return;
      }
      option.label = option.label.substring(this.SCRIPT_DEFINED_MARK.length);
      selectWidget.updateOption(option);
    },

    isMarkSetOptionLabel: function(option) {
      if (!option || !option.label) {
        return false;
      }
      if (option.label.lastIndexOf(this.SCRIPT_DEFINED_MARK, 0) === 0) {
        return true;
      }
      return false;
    },

    getModelDescription: function(modelId, callback) {
      this.project.getModelDescriptionWithId(modelId).then(function(description) {
        if (!description) {
          description = modelId;
        }
        callback(description);
      });
    },

    getModelitemLabelWithName: function(modelId, modelitemName, callback) {
      this.project.getModelitemLabelWithName(modelId, modelitemName).then(function(label) {
        if (!label) {
          label = modelitemName;
        }
        callback(label);
      });
    },

    showConfirmDialog: function(args) {
      if (this.confirmDialog) {
        this.confirmDialog.updateMessage(args.message);
        this.confirmDialog.setEventCallbackFunc(args);
        if (args.size) {
          this.confirmDialog.resizeDialog(args.size);
        }
        this.confirmDialog.show();
      } else {
        var that = this;
        this.confirmDialog = this.createConfirmDialog({
          "message": args.message || "",
          "size": args.size,
          "onExecute": args.onExecute,
          "onCancel": args.onCancel,
          "onShow": args.onShow,
          "onHide": args.onHide,
        }, function(dialog) {
          that.confirmDialog = dialog;
          that.confirmDialog.show();
        });
      }
    },

    getVersionArray: function(version) {
      var versionArray = [];
      var versions = version.split(".");
      if (!versions || versions.length !== 3) {
        return versionArray;
      }
      versionArray.push(parseInt(versions[0], 10));
      versionArray.push(parseInt(versions[1], 10));
      versionArray.push(parseInt(versions[2], 10));
      return versionArray;
    },

    setWdVersion: function(data) {
      if (!data) {
        return;
      }
      var versions = data.split(" ");
      if (!versions || versions.length !== 3) {
        return;
      }
      var version = versions[1].replace("R", "");
      if (!version || version === "") {
        return;
      }
      this.wdVersion = version;
      var versionArray = this.getVersionArray(version);
      if (!versionArray || versionArray.length !== 3) {
        return;
      }
      this.wdMajorVersion = versionArray[0];
      this.wdMinorVersion = versionArray[1];
      this.wdTinyVersion = versionArray[2];
    },

    setUpdateRowSelectors: function(grid) {
      grid._updateRowSelectors = function (value, event) {
        var rows = event.rows;
        for (var iRows = 0, lenRows = rows.length; iRows < lenRows; iRows++) {
          for (var iCols = 0, lenCols = this._selectorColumns.length; iCols < lenCols; iCols++) {
            var column = this._selectorColumns[iCols];
            var element = this.cell(rows[iRows], column.id).element;
            if (!element || !element.childNodes || element.childNodes.length === 0) {
              continue;
            }
            // 行選択の dijit/form/CheckBox のノードを取得する
            element = element.childNodes[0].childNodes[0];
            if (!element || element.disabled) {
              continue;
            }
            if (!element.id) {
              continue;
            }
            var widget = registry.byId(element.id);
            if (!widget) {
              continue;
            }
            widget.set('checked', value);
          }
        }
      };

      grid._updateHeaderCheckboxes = function () {
        var lenCols = this._selectorColumns.length;
        for (var iCols = 0,l=lenCols; iCols < l; iCols++) {
          var column = this._selectorColumns[iCols];
          var state = 'false';
          var selection;
          var mixed;
          var selectorHeaderCheckbox = column._selectorHeaderCheckbox;
          if (selectorHeaderCheckbox) {
            selection = this.selection;
            mixed = false;
            for (var i in selection) {
              if (selection[i] != this.allSelected) {
                mixed = true;
                break;
              }
            }
            var selectorHeaderCheckboxWidget = registry.byId(selectorHeaderCheckbox.id);
            if (!selectorHeaderCheckboxWidget) {
              continue;
            }
            if (this.allSelected && mixed) {
              // チェックボックス全選択後に個別にチェックを外した場合、ヘッダのチェックボックスを外す
              selectorHeaderCheckboxWidget.set('checked', false);
              this.allSelected = false;
            } else if (!this.allSelected && Object.keys(selection).length === this.get('total')) {
              // 個別のチェックを付けて全選択状態になった場合、ヘッダのチェックボックスを付ける
              selectorHeaderCheckboxWidget.set('checked', true);
              this.allSelected = true;
            }
          }
        }
      };
    },

    getSettigns: function() {
      var that = this;
      wagbyDesigner.sendRequest({
        url: "/wd/settings",
        options: {
          handleAs: "json"
        }
      }, function(data) {
        that.settings = data;
        that.setBuildMenuDisplayAttr();
      }, function(error) {
        return that.jfcRes.MSG_ERROR_WAGBYDESIGNER_SETTINGS_001; // "WagbyDesigner設定の取得に失敗しました。"
      });
    },

    setBuildMenuDisplayAttr: function() {
      var display = "";
      var buildDisable = this.settings["build.disable"];
      if (buildDisable && buildDisable.toLowerCase() === "true") {
        display = "none";
      }
      domStyle.set("menuButtonBuild", "display", display);
    },

    getSettings: function() {
      return this.settings;
    },

    getPrincipalData: function(callback) {
      if (!this.principalData) {
        var that = this;
        this.getPrincipals(function(data) {
          that.createPrincipalData(data);
          that.executePrincipalCallback(callback);
        });
      } else {
        this.executePrincipalCallback(callback);
      }
    },

    getPrincipals: function(callback) {
      var that = this;
      this.sendRequest({
        url: wagbyDesigner.BASEURL + "/authority/principals",
        options: {
          handleAs: "json",
          headers: {
            'Content-Type': "application/json; charset=utf-8"
          }
        }
      }, function(data) {
        callback(data);
      }, function(data) {
        return that.jfcRes.MSG_ERROR_AUTHORITY_PRINCIPAL_001;
      });
    },

    createPrincipalData: function(principals) {
      this.principalData = {
        system: [],
        user: [],
      };
      for (var i=0,l=principals.length; i<l; i++) {
        var principal = principals[i];
        var data = {
          'id': principal.id,
          'name': this.getPrincipalNameByLocale(principal),
          'no': principal.no,
          'order': principal.order,
        };
        this.principalData.system.push(data);
        if (principal.isShowInSelect) {
          this.principalData.user.push(data);
        }
      }
    },

    executePrincipalCallback: function(callback) {
      var data;
      if (!this.menuPane.isMenuPaneDisplayed && this.modelPane.isSystemModel()) {
        data = this.principalData.system;
      } else {
        data = this.principalData.user;
      }
      callback(data);
    },

    clearPrincipalData: function() {
      if (!this.principalData) {
        return;
      }
      this.principalData.system = null;
      this.principalData.user = null;
      this.principalData = null;
    },

    getPrincipalNameByLocale: function(principal) {
      return this.getResourceByLocale(principal.content) + "(" + principal.principalId + ")";
    },

    getResourceByLocale: function(content) {
      var resource = JSON.parse(content);
      return resource[this.locale] || resource["en"];
    },

    setGridStyle: function(grid, columnStyles) {
      for (var key in columnStyles) {
        domStyle.set(grid.columns[key].headerNode, columnStyles[key]);
      }
    },

    getSelectedId: function(grid) {
      var selectedId = [];
      for (var key in grid.selection) {
        selectedId.push(key);
      }
      return selectedId;
    },

    resizeGridHeight: function(grid) {
      // グリッドの高さをブラウザウィンドウに合わせてセットする
      var offsetHeight = 200;
      var windowHeight = win.getBox().h;
      if (windowHeight < 700) {
        offsetHeight = 180;
      }
      var height = windowHeight - offsetHeight;
      domStyle.set(grid.domNode, 'height', height + "px");
      grid.resize();
    },

    setGridColumnStyle: function(rowElement, gridColumns) {
      for (var i=0,l=gridColumns.length; i<l; i++) {
        var column = gridColumns[i];
        if (!column.style) {
          continue;
        }
        query("td.dgrid-column-" + column.id, rowElement).forEach(function(node) {
          domStyle.set(node, column.style);
        });
      }
    },

    sendData: function(args) {
      var headers = {
        'Content-Type': 'application/json; charset=utf-8',
      };
      if (args.headers) {
        for (var key in args.headers) {
          if (args.headers[key]) {
            headers[key] = args.headers[key];
          }
        }
      }
      var that = this;
      this.sendRequest({
        url: that.BASEURL + args.url,
        options: {
          handleAs: "json",
          headers: headers,
          data: args.data,
          method: args.method || "GET",
        }
      }, function(data) {
        if (args.callback) {
          args.callback(data);
        }
      }, function(data) {
console.error("send data error : ", data);
        if (args.errorMessage) {
          return args.errorMessage;
        }
        return that.jfcRes.MSG_ERROR_SEND_REQUEST;
      });
    },

    getIdFromDroppedNodes: function(nodes) {
      var id = [];
      for (var i=0,l=nodes.length; i<l; i++) {
        var node = nodes[i];
        if (!node.id) {
          continue;
        }
        var index = node.id.lastIndexOf("-");
        if (index === -1) {
          continue;
        }
        id.push(node.id.substr(index + 1));
      }
      return id;
    },

    getDomains: function(callback) {
      var that = this;
      this.sendRequest({
        url: this.BASEURL + "/domains",
        options: {
          handleAs: "json",
        }
      }, function(data) {
        that.setTargetDomainSelectOptions(data);
        if (callback) {
          callback(data);
        }
      }, function(error) {
console.log("getDomains error: ", error);
        return this.jfcRes.MSG_ERROR_DOMAIN_001;
      });
    },

    getUserDomainOptions: function(domains) {
      var userDomains = [{
        label: " ",
        value: 0,
      }];
      if (!this.isMicroserviceEnabled()) {
        return userDomains;
      }
      return userDomains.concat(array.filter(domains, function(domain) {
        return domain.id !== parseInt(wagbyDesigner.DOMAIN_SYSTEM_ID, 10) &&
          domain.id !== parseInt(wagbyDesigner.DOMAIN_COMMON_ID, 10);
      }).map(function(domain) {
        return {
          label: domain.domainName,
          value: domain.id,
        };
      }, this));
    },

    setTargetDomainSelectOptions: function(data) {
      var selectedDomainId = this.targetDomainSelect.get("value");
      var userDomains = this.getUserDomainOptions(data.domains);
      if (userDomains.length > 1) {
        userDomains.shift();
      }
      this.targetDomainSelect.set("options", userDomains);
      var value;
      if (userDomains.length === 1 && userDomains[0].value === 0) {
        value = 0;
      } else {
        if (selectedDomainId && selectedDomainId !== 0) {
          value = selectedDomainId;
        } else {
          value = userDomains[0].value;
        }
      }
      this.targetDomainSelect.set("value", value);
    },

    isMicroserviceEnabled: function() {
      return this.projectEnvironmentPane.isMicroserviceEnabled();
    },

    refreshBorderContainer: function() {
      if (!this.borderContainer) {
        return;
      }
      this.borderContainer.layout();
    },

    displayMicroserviceSetting: function(isMicroserviceEnabled) {
      this.projectEnvironmentPane.setCommonMarkToLabel(isMicroserviceEnabled);
      this.changeTargetDomainSelectOptions(isMicroserviceEnabled);
      this.setTargetDomainSelectDisplay(isMicroserviceEnabled);
    },

    setTargetDomainSelectDisplay: function(isDisplay) {
      var display = "none";
      if (isDisplay) {
        display = "";
      }
      domStyle.set(this.targetDomainSelectArea, "display", display);
    },

    changeTargetDomainSelectOptions: function(value) {
      var options = this.targetDomainSelect.get("options");
      if (!options || options.length === 0) {
        return;
      }
      this.targetDomainSelect.set("value", options[0].value);
    },

    getTargetDomainId: function() {
      return wagbyDesigner.targetDomainSelect.get("value");
    },

    getDomainWidgetOnClickFunc: function(widgets) {
      var that = this;
      return function() {
        that.exclusiveDomainSettings(that.getCheckedDomainWidgetValues(widgets), widgets);
      }
    },

    getCheckedDomainWidgetValues: function(widgets) {
      var values = [];
      for (var i=0,l=widgets.length; i<l; i++) {
        var widget = widgets[i];
        if (!widget.get("checked")) {
          continue;
        }
        values.push(widget.get("value"));
      }
      return values;
    },

    exclusiveDomainSettings: function(values, widgets) {
      if (values.length === 0) {
        this.enableAllDomainWidgets(widgets);
      } else if (values.includes(this.DOMAIN_COMMON_ID)) {
        this.enableCommonDomainWidgets(widgets);
      } else {
        this.enableUserDomainWidgets(widgets)
      }
    },

    setDomainWidgetsProperties: function(widgets, conditionFunc) {
      for (var i=0,l=widgets.length; i<l; i++) {
        var widget = widgets[i];
        if (conditionFunc(widget)) {
          widget.set("disabled", false);
        } else {
          widget.set("disabled", true);
          widget.set("checked", false);
        }
      }
    },

    enableAllDomainWidgets: function(widgets) {
      this.setDomainWidgetsProperties(widgets, function(widget) {
        return true;
      })
    },

    enableCommonDomainWidgets: function(widgets) {
      var that = this;
      this.setDomainWidgetsProperties(widgets, function(widget) {
        return widget.value === that.DOMAIN_COMMON_ID;
      })
    },

    enableUserDomainWidgets: function(widgets) {
      var that = this;
      this.setDomainWidgetsProperties(widgets, function(widget) {
        return widget.value !== that.DOMAIN_COMMON_ID;
      })
    },

    getGridStore: function(idProperty) {
      var store = new (declare([Memory, Trackable]))({
        idProperty: idProperty,
        data: [],
      });
      return store;
    },

    createGrid: function(gridId, gridColumns, store, sortProps) {
      var grid = new (declare([OnDemandGrid, Editor, DnD, Keyboard, Selector]))({
        columns: gridColumns,
        allowSelectAll: true,
        autoSave: true,
        autoSelect: true,
        selectionMode: 'none',
        insertRow: function () {
          var rowElement = this.inherited(arguments);
          wagbyDesigner.setGridColumnStyle(rowElement, gridColumns);
          return rowElement;
        },
      }, gridId);
      grid.startup();
      grid.set("collection", store);
      if (sortProps) {
        grid.set('sort', sortProps);
      }
      grid.dndSource.withHandles = true;
      // 行選択チェックボックスの関数をカスタマイズする
      wagbyDesigner.setUpdateRowSelectors(grid);
      return grid;
    },

    getErrorMessage: function(error, message) {
      var errorMessage = message;
      if (!error || !error.response || !error.response.data) {
        return errorMessage;
      }
      var errorData = JSON.parse(error.response.data);
      if (errorData && errorData.message) {
        errorMessage = errorData.message;
      }
      return errorMessage;
    },

    sendJsonData: function(args) {
      var headers = {
        'Content-Type': 'application/json; charset=utf-8',
      };
      if (args.headers) {
        for (var key in args.headers) {
          if (args.headers[key]) {
            headers[key] = args.headers[key];
          }
        }
      }
      var that = this;
      this.sendRequest({
        url: this.BASEURL + args.url,
        options: {
          handleAs: "json",
          headers: headers,
          data: args.data,
          method: args.method || "GET",
        }
      }, function(data) {
        if (args.callback) {
          args.callback(data);
        }
      }, function(data) {
console.error("send data error : ", data);
        return args.errorMessage;
      });
    },

    showErrorResponse: function(response) {
      var msg = wagbyDesigner.jfcRes.MSG_ERROR_SAVE_REPOSITORY_DATA;
      if (response && response.status === 409) {
        msg = wagbyDesigner.jfcRes.MSG_ERROR_SAVE_REPOSITORY_DATA_LOCKED;
      }
      wagbyDesigner.showDialog(msg);
    },

    getEllipsisString: function(value, len) {
      if (!value || value.length < len) {
        return value;
      }
      return value.substring(0, len + 1) + "...";
    },

  });
});
